function plugindef()
   -- This function and the 'finaleplugin' namespace
   -- are both reserved for the plug-in definition.
   finaleplugin.RequireSelection = true
   finaleplugin.Author = "Chris"
   finaleplugin.Version = "0.9"
   finaleplugin.Date = "27/04/2015"
   return "- Repeat 4 mesures", "Repeat 4 mesures", ""
end

local NAME_SHAPE_EXP = "Shape Repeat 4"

function caractere(instructions,char,x,y,xlt,ylt,xrb,yrb)
    local origin =finale.FCPoint(x,y)
    local lefttop =finale.FCPoint(xlt,ylt)
    local rightbottom =finale.FCPoint(xrb,yrb)
    instructions:AddStartObject(origin,lefttop,rightbottom,1000,1000,0)

    local font = finale.FCFontInfo()
    font.Name="Maestro"
    font.Size=24
    font.Bold=false
    font.Italic=false
    instructions:AddSetFont(font)

    instructions:AddDrawChar(char)
    instructions:AddStroke()
end

function creerShape()
    -- retourne l'ID de la shape
    local shapedef =finale.FCShapeDef()
    local instructions = shapedef:CreateInstructions()
    caractere(instructions,52,38,30,38,170,78,-64)  -- chiffre 4
    caractere(instructions,46,13,-38,13,102,27,-132)  -- point gauche
    caractere(instructions,46,99,-62,99,78,113,-156)  -- point droit
    caractere(instructions,213,3,-73,3,67,67,-167)  -- slash gauche
    caractere(instructions,213,24,-73,24,67,88,-167)  -- 2° slash
    caractere(instructions,213,45,-73,45,67,110,-167)  -- 3° slash
    caractere(instructions,213,66,-73,66,67,131,-167)  -- 4° slash

    instructions:AddNull()
 
    -- sauvegarde
    shapedef:RebuildInstructions(instructions)
    shapedef:SaveNewWithType(-1)

    return shapedef.ItemNo
end

function creerExpression(idShape)
    local sed = finale.FCShapeExpressionDef()
    sed.ID = idShape
    sed.HorizontalJustification = finale.EXPRJUSTIFY_CENTER 
    sed.HorizontalAlignmentPoint = finale.ALIGNHORIZ_RIGHTBARLINE 
    sed.VerticalAlignmentPoint = finale.ALIGNVERT_STAFF_REFERENCE_LINE 
    sed.AllowHorizontalStretching = false

    local str = finale.FCString()
    str.LuaString = NAME_SHAPE_EXP
    sed:SetDescription(str)

    sed:SaveNew()

    return sed.ItemNo
 end

function existExpression()
    local shapeexpressiondefs = finale.FCShapeExpressionDefs()
    shapeexpressiondefs:LoadAll()

    for e in each(shapeexpressiondefs) do
        local description = finale.FCString()
        e:GetDescription(description)
        if description:IsEqual(NAME_SHAPE_EXP) then
            return e.ItemNo
        end
    end

    return nil
end

function existSmartLine()
    -- teste si la smartline "repeat 4" est chargée et retourne l'index
    local customsmartlinedefs = finale.FCCustomSmartLineDefs()
    customsmartlinedefs:LoadAll()

    for csld in each(customsmartlinedefs) do
        if testEgalSmartLine(csld) then
            return csld.ItemNo
        end                                      
    end

    return -1
end

function testEgalSmartLine(csld)
    if csld.Horizontal ~= true then return false end
    if csld.LineStyle ~= finale.CUSTOMLINE_SOLID then return false end
    if csld.StartArrowheadStyle ~= finale.CLENDPOINT_HOOK  then return false end
    if csld.StartHookLength ~= -1451 then return false end
    if csld.EndArrowheadStyle ~= finale.CLENDPOINT_HOOK  then return false end
    if csld.EndHookLength ~= -1451 then return false end
    if csld.LineWidth ~= 363 then return false end

    local text = csld:CreateTextString(finale.TEXTREF_LEFTSTART)
    if text ~= nil then return false end
    local text = csld:CreateTextString(finale.TEXTREF_LEFTCONTINUATION)
    if text ~= nil then return false end
    local text = csld:CreateTextString(finale.TEXTREF_RIGHTEND)
    if text ~= nil then return false end
    local text = csld:CreateTextString(finale.TEXTREF_CENTERFULL)
    if text ~= nil then return false end
    local text = csld:CreateTextString(finale.TEXTREF_CENTERABBREVIATED)
    if text ~= nil then return false end

    return true
end

function creerLigne()
    local csld = finale.FCCustomSmartLineDef()
    csld.Horizontal =true
    csld.LineStyle = finale.CUSTOMLINE_SOLID
    csld.StartArrowheadStyle = finale.CLENDPOINT_HOOK 
    csld.StartHookLength = -1451
    csld.EndArrowheadStyle = finale.CLENDPOINT_HOOK 
    csld.EndHookLength = -1451
    csld.LineWidth = 363
 
    csld:SaveNew()

    return csld.ItemNo
end

function longueurMesure(ind)
    local m = finale.FCMeasure()
    if m:Load(ind) then
        return m:GetDuration()
    end
    return 4096
end

function creerSmartShape(staff,mesure,indLigne)
    local smartshape = finale.FCSmartShape()
    smartshape.ShapeType = finale.SMARTSHAPE_CUSTOM
    smartshape.EntryBased =false
    smartshape.MakeHorizontal =true
    smartshape.BeatAttached= true
    smartshape.Visible = true
    smartshape:SetPresetShape(true)
    smartshape.LineID=indLigne

    local leftseg = smartshape:GetTerminateSegmentLeft()
    leftseg:SetMeasure(mesure)
    leftseg:SetMeasurePos(0)
    leftseg:SetStaff(staff)
    leftseg:SetEndpointOffsetX(-14)
    leftseg:SetEndpointOffsetY(-13)

    local indMesFin = mesure + 3
    local rightseg = smartshape:GetTerminateSegmentRight()
    rightseg:SetMeasure(indMesFin)
    rightseg:SetMeasurePos(longueurMesure(indMesFin))
    rightseg:SetStaff(staff)
    rightseg:SetEndpointOffsetX(-19)
    rightseg:SetEndpointOffsetY(-13)

    smartshape:SaveNewEverything(nil,nil)
end

-- DEBUT DU PROGRAMME
local idShapeExpression=existExpression() 
if idShapeExpression == nil then
    local idShape = creerShape() 
    idShapeExpression = creerExpression(idShape)
end

local idSmartShape = existSmartLine()
if idSmartShape == -1 then idSmartShape = creerLigne() end

local region = finale.FCMusicRegion()
region:SetCurrentSelection()

for slot = region.StartSlot, region.EndSlot do    
    local staffnumber = region:CalcStaffNumber(slot)

    for mesure = region.StartMeasure,region.EndMeasure,4 do
        local e = finale.FCExpression()
        e.Shape = true
        e.ID = idShapeExpression
        e:SaveNewToCell(finale.FCCell(mesure + 1,staffnumber))

        creerSmartShape(staffnumber,mesure,idSmartShape)
    end
end